/* eslint-disable react/jsx-handler-names */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import FirstPageLink from './FirstPageLink';
import NextPageLink from './NextPageLink';
import PrevPageLink from './PrevPageLink';
import LastPageLink from './LastPageLink';
import RowsPerPageDropdown from './RowsPerPageDropdown';
import CurrentPageReport from './CurrentPageReport';
import PageLinks from './PageLinks';

export class Paginator extends Component {
	constructor(props) {
		super(props);
		this.changePageToFirst = this.changePageToFirst.bind(this);
		this.changePageToPrev = this.changePageToPrev.bind(this);
		this.changePageToNext = this.changePageToNext.bind(this);
		this.changePageToLast = this.changePageToLast.bind(this);
		this.onRowsChange = this.onRowsChange.bind(this);
		this.onPageLinkClick = this.onPageLinkClick.bind(this);
	}

	isFirstPage() {
		return this.getPage() === 0;
	}

	isLastPage() {
		return this.getPage() === this.getPageCount() - 1;
	}

	getPageCount() {
		const { rows, totalRecords } = this.props;
		return Math.ceil(totalRecords / rows) || 1;
	}

	calculatePageLinkBoundaries() {
		const { pageLinkSize } = this.props;
		const numberOfPages = this.getPageCount();
		const visiblePages = Math.min(pageLinkSize, numberOfPages);

		// calculate range, keep current in middle if necessary
		let start = Math.max(0, Math.ceil(this.getPage() - visiblePages / 2));
		const end = Math.min(numberOfPages - 1, start + visiblePages - 1);

		// check when approaching to last page
		const delta = pageLinkSize - (end - start + 1);
		start = Math.max(0, start - delta);

		return [start, end];
	}

	updatePageLinks() {
		const pageLinks = [];
		const boundaries = this.calculatePageLinkBoundaries();
		const start = boundaries[0];
		const end = boundaries[1];

		for (let i = start; i <= end; i++) {
			pageLinks.push(i + 1);
		}

		return pageLinks;
	}

	changePage(first, rows) {
		const { onPageChange } = this.props;
		const pc = this.getPageCount();
		const p = Math.floor(first / rows);

		if (p >= 0 && p < pc) {
			const newPageState = {
				first,
				rows,
				page: p,
				pageCount: pc,
			};

			if (onPageChange) {
				onPageChange(newPageState);
			}
		}
	}

	getPage() {
		const { first, rows } = this.props;
		return Math.floor(first / rows);
	}

	changePageToFirst(event) {
		const { rows } = this.props;
		this.changePage(0, rows);
		event.preventDefault();
	}

	changePageToPrev(event) {
		const { first, rows } = this.props;
		this.changePage(first - rows, rows);
		event.preventDefault();
	}

	onPageLinkClick(event) {
		const { rows } = this.props;
		this.changePage((event.value - 1) * rows, rows);
	}

	changePageToNext(event) {
		const { first, rows } = this.props;
		this.changePage(first + rows, rows);
		event.preventDefault();
	}

	changePageToLast(event) {
		const { rows } = this.props;
		this.changePage((this.getPageCount() - 1) * rows, rows);
		event.preventDefault();
	}

	onRowsChange(event) {
		this.changePage(0, event.value);
	}

	render() {
		const {
			alwaysShow,
			className,
			currentPageReportTemplate,
			leftContent,
			rightContent,
			rows,
			rowsPerPageOptions,
			style,
			template,
		} = this.props;
		if (!alwaysShow && this.getPageCount() === 1) {
			return null;
		} else {
			const classNameCalc = classNames('p-paginator p-component p-unselectable-text', className);

			const paginatorElements = template.split(' ').map(value => {
				const key = value.trim();
				let element;

				switch (key) {
					case 'FirstPageLink':
						element = (
							<FirstPageLink
								key={key}
								onClick={this.changePageToFirst}
								disabled={this.isFirstPage()}
							/>
						);
						break;

					case 'PrevPageLink':
						element = (
							<PrevPageLink
								key={key}
								onClick={this.changePageToPrev}
								disabled={this.isFirstPage()}
							/>
						);
						break;

					case 'NextPageLink':
						element = (
							<NextPageLink
								key={key}
								onClick={this.changePageToNext}
								disabled={this.isLastPage()}
							/>
						);
						break;

					case 'LastPageLink':
						element = (
							<LastPageLink
								key={key}
								onClick={this.changePageToLast}
								disabled={this.isLastPage()}
							/>
						);
						break;

					case 'PageLinks':
						element = (
							<PageLinks
								key={key}
								value={this.updatePageLinks()}
								page={this.getPage()}
								onClick={this.onPageLinkClick}
							/>
						);
						break;

					case 'RowsPerPageDropdown':
						element = (
							<RowsPerPageDropdown
								key={key}
								value={rows}
								options={rowsPerPageOptions}
								onChange={this.onRowsChange}
							/>
						);
						break;

					case 'CurrentPageReport':
						element = (
							<CurrentPageReport
								template={currentPageReportTemplate}
								key={key}
								page={this.getPage()}
								pageCount={this.getPageCount()}
							/>
						);
						break;

					default:
						element = null;
						break;
				}

				return element;
			});

			const leftContentCalc = leftContent && (
				<div className="p-paginator-left-content">{leftContent}</div>
			);
			const rightContentCalc = rightContent && (
				<div className="p-paginator-right-content">{rightContent}</div>
			);

			return (
				<div className={classNameCalc} style={style}>
					{leftContentCalc}
					{paginatorElements}
					{rightContentCalc}
				</div>
			);
		}
	}
}

Paginator.defaultProps = {
	totalRecords: 0,
	rows: 0,
	first: 0,
	pageLinkSize: 5,
	rowsPerPageOptions: null,
	style: null,
	className: null,
	template: 'FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink RowsPerPageDropdown',
	onPageChange: null,
	leftContent: null,
	rightContent: null,
	currentPageReportTemplate: '({currentPage} of {totalPages})',
	alwaysShow: true,
};

Paginator.propTypes = {
	alwaysShow: PropTypes.bool,
	className: PropTypes.string,
	currentPageReportTemplate: PropTypes.any,
	first: PropTypes.number,
	leftContent: PropTypes.any,
	onPageChange: PropTypes.func,
	pageLinkSize: PropTypes.number,
	rightContent: PropTypes.any,
	rows: PropTypes.number,
	rowsPerPageOptions: PropTypes.array,
	style: PropTypes.object,
	template: PropTypes.string,
	totalRecords: PropTypes.number,
};

export default Paginator;
